<?php

namespace Drupal\transform_api\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\transform_api\EntityTransformer;
use Drupal\transform_api\Repository\EntityTransformRepositoryInterface;
use Drupal\transform_api\Transform\EntityTransform;
use Drupal\transform_api\Transformer;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

class EntityController extends \Drupal\Core\Controller\ControllerBase {
  protected $transformer;
  protected $entityTypeManager;
  protected $configFactory;

  function __construct(Transformer $transformer, EntityTypeManagerInterface $entityTypeManager, ConfigFactoryInterface $configFactory) {
    $this->transformer = $transformer;
    $this->entityTypeManager = $entityTypeManager;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('transform_api.transformer'),
      $container->get('entity_type.manager'),
      $container->get('config.factory')
    );
  }

  /**
   * @param $entity_type
   * @param $id
   * @param $transform_mode
   * @param $langcode
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   */
  public function view($entity_type, $id, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE, $langcode = ''): JsonResponse {
    $transform = new EntityTransform($entity_type, $id, $transform_mode);
    return new JsonResponse($this->transformer->transformRoot($transform));
  }

}
