<?php

namespace Drupal\transform_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\transform_api\Exception\ResponseTransformationException;
use Drupal\transform_api\Transform\RequestPathTransform;
use Drupal\transform_api\Transformer;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;

class RequestPathController extends ControllerBase {
  protected Transformer $transformer;
  protected RequestStack $requestStack;

  function __construct(Transformer $transformer, RequestStack $requestStack) {
    $this->transformer = $transformer;
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('transform_api.transformer'),
      $container->get('request_stack')
    );
  }

  public function enhance($url) {
    try {
      $transform = new RequestPathTransform($url);
      return new JsonResponse($this->transformer->transformRoot($transform));
    } catch (ResponseTransformationException $exception) {
      return $exception->getResponse();
    }
  }

  public function enhanceRegion($url, $region) {
    $transform = new RequestPathTransform($url, $region);
    return new JsonResponse($this->transformer->transformRoot($transform));
  }

}
