<?php

namespace Drupal\transform_api;

use Drupal\Component\Plugin\DerivativeInspectionInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\PluginSettingsInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;

interface FieldTransformInterface extends PluginInspectionInterface, DerivativeInspectionInterface, ContainerFactoryPluginInterface, PluginSettingsInterface {

  /**
   * Returns the view mode the plugin is configured for.
   *
   * @return string
   */
  public function getTransformMode();

  /**
   * Allows formatters to load information for field values being displayed.
   *
   * This should be used when a formatter needs to load additional information
   * from the database in order to render a field, for example a reference
   * field that displays properties of the referenced entities such as name or
   * type.
   *
   * This method operates on multiple entities. The $entities_items parameter
   * is an array keyed by entity ID. For performance reasons, information for
   * all involved entities should be loaded in a single query where possible.
   *
   * Changes or additions to field values are done by directly altering the
   * items.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface[] $entities_items
   *   An array with the field values from the multiple entities being rendered.
   */
  public function prepareTransform(array $entities_items);

  public function transform(FieldItemListInterface $items, $langcode = NULL);

  public function transformElements(FieldItemListInterface $items, $langcode);

  /**
   * Returns if the transformer can be used for the provided field.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition that should be checked.
   *
   * @return bool
   *   TRUE if the transformer can be used, FALSE otherwise.
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition);

}
