<?php

namespace Drupal\transform_api\Plugin\Transform\Type;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\transform_api\Transform\TransformInterface;
use Drupal\transform_api\TransformationTypeBase;
use Drupal\transform_api\TransformBlockManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @TransformationType(
 *  id = "block",
 *  title = "Block transform"
 * )
 */
class Block extends TransformationTypeBase {

  /**
   * @var ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * @var EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  protected TransformBlockManager $blockTransformManager;

  function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $configFactory, ModuleHandlerInterface $moduleHandler, TransformBlockManager $blockTransformManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $configFactory;
    $this->moduleHandler = $moduleHandler;
    $this->blockTransformManager = $blockTransformManager;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('config.factory'),
      $container->get('module_handler'),
      $container->get('plugin.manager.transform_api.transform_block')
    );
  }

  public function transform(TransformInterface $transform) {
    /** @var \Drupal\transform_api\Transform\BlockTransform $block_transform */
    $block_transform = $transform;
    $config = $block_transform->getBlock();
    $plugin = $config->getPlugin();
    $cache = CacheableMetadata::createFromObject($plugin);
    $cache->addCacheableDependency($config);
    $transformation = array_merge(['#block_id' => $transform->getValue('id')], $plugin->transform());
    $cache->addCacheableDependency(CacheableMetadata::createFromRenderArray($transformation));
    $cache->applyTo($transformation);
    return $transformation;
  }

}
