<?php

namespace Drupal\transform_api\Plugin\Transform\Type;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\transform_api\RouteTransformManager;
use Drupal\transform_api\Transform\TransformInterface;
use Drupal\transform_api\TransformationTypeBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @TransformationType(
 *  id = "route",
 *  title = "Route"
 * )
 */
class Route extends TransformationTypeBase {

  /**
   * @var \Drupal\transform_api\RouteTransformManager
   */
  protected RouteTransformManager $routeTransformManager;

  function __construct(array $configuration, $plugin_id, $plugin_definition, RouteTransformManager $routeTransformManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->routeTransformManager = $routeTransformManager;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('plugin.manager.transform_api.route_transform')
    );
  }

  public function transform(TransformInterface $transform) {
    try {
      $route = $this->routeTransformManager->createInstance($transform->getValue('route_name'));
    } catch (PluginException $e) {
      return [];
    }
    if (empty($route)) {
      return [];
    }

    $transformation = [
        'type' => 'route',
      ] + $transform->getValues();
    $transformation += $route->transform($transform);

    return $transformation;
  }
}
