<?php

namespace Drupal\transform_api\Transform;

use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Cache\CacheableMetadata;

interface TransformInterface extends CacheableDependencyInterface {

  /**
   * @return string
   */
  public function getTransformType();

  /**
   * @return string[]
   */
  public function getAlterIdentifiers();

  /**
   * @return string[]
   */
  public function getValues();


  /**
   * @param string $key
   *
   * @return mixed
   */
  public function getValue($key);

  /**
   * @param $values
   *
   * @return void
   */
  public function setValues($values);

  /**
   * @param $key
   * @param $value
   *
   * @return void
   */
  public function setValue($key, $value);

  /**
   * The cache keys associated with this object.
   *
   * @return string[]
   *   A set of cache keys.
   */
  public function getCacheKeys();

  /**
   * The weight of the transform in case of sorting;
   *
   * @return int|null
   */
  public function getWeight();

  /**
   * Perform the transformation and return it as an array ready to be JSON encoded.
   * Can include caching metadata and other metadata if the key starts with "#".
   *
   * In case the transform is actually multiple transforms wrapped in one, the returned
   * array should be an array of transforms.
   *
   * @return array
   *
   * @throws \Drupal\transform_api\TransformationException
   */
  public function transform();

  /**
   * Whether it is worth caching the transformation independently or not.
   * Even if FALSE is returned, the transformation can still be cached as part of a greater transformation.
   * In cases where the transformation already holds all the data this is a good option.
   *
   * If a transformation should never be cached, instead set cache max age to 0.
   *
   * @return bool
   */
  public function shouldBeCached();

  /**
   * @return bool
   */
  public function isMultiple();
}
